SystemOrganization addCategory: #Mass!
SystemOrganization addCategory: 'Mass-DESimEngine'!
SystemOrganization addCategory: 'Mass-Projection'!
SystemOrganization addCategory: 'Mass-SimST80Demos'!
SystemOrganization addCategory: 'Mass-Statistics'!

Stream subclass: #ProbabilityDistribution
	instanceVariableNames: ''
	classVariableNames: 'U'
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!ProbabilityDistribution commentStamp: '<historical>' prior: 0!
Initialization of the class is needed.  Use

        ProbabilityDistribution initialize!

ProbabilityDistribution subclass: #ContinuousProbability
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!ContinuousProbability methodsFor: 'probability functions' stamp: ''!
distribution: aCollection
        "This is a slow and dirty trapezoidal integration to determine the area under the probability function curve y=density (x) for x in the specified collection.  The method assumes that the collection contains numerically-ordered elements."

        | t aStream x1 x2 y1 y2 |
        t := 0.0.
        aStream := ReadStream on: aCollection.
        x2 := aStream next.
        y2 := self density: x2.
        [x1 := x2.  x2 := aStream next]
                whileTrue:  
                        [y1 := y2.
                         y2 := self density: x2.
                         t := t + ((x2-x1)*(y2+y1))].
        ^t*0.5! !

ContinuousProbability subclass: #Exponential
	instanceVariableNames: 'mu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Exponential class methodsFor: 'instance creation' stamp: ''!
mean: p
        ^self parameter: 1.0/p! !

!Exponential class methodsFor: 'instance creation' stamp: ''!
parameter:  p
         
        p > 0.0
                ifTrue: [^self new setParameter: p]
                ifFalse: [self error: 'The probability parameter must be greater than 0.0']! !

!Exponential methodsFor: 'probability functions' stamp: ''!
density: x
        x > 0.0
                ifTrue: [^mu * (mu*x) negated exp]
                ifFalse: [^0.0]! !

!Exponential methodsFor: 'probability functions' stamp: ''!
distribution: anInterval
        anInterval last <= 0.0
                ifTrue: [^0.0]
                ifFalse: [^1.0 - (mu * anInterval last) negated exp - (anInterval first > 0.0 ifTrue: [self distribution: (0.0 to: anInterval first)] ifFalse: [0.0])]! !

!Exponential methodsFor: 'private' stamp: ''!
inverseDistribution: x
        ^ x ln negated / mu! !

!Exponential methodsFor: 'accessing' stamp: ''!
mean
        ^1.0/mu! !

!Exponential methodsFor: 'private' stamp: ''!
setParameter: p
        mu := p! !

!Exponential methodsFor: 'accessing' stamp: ''!
variance
        ^1.0/(mu*mu)! !

Exponential subclass: #Gamma
	instanceVariableNames: 'N'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Gamma class methodsFor: 'instance creation' stamp: ''!
events: k mean: p

        |  events |
        events := k truncated.
        events > 0
                ifTrue: [^(self parameter: events/p) setEvents: events]
                ifFalse: [self error: 'the number of events must be greater than 0']! !

!Gamma methodsFor: 'probability functions' stamp: ''!
density: x
        | t |
        x > 0.0
                ifTrue: [t := mu * x.
                        ^(mu raisedTo: N) / (self gamma: N) *(x raisedTo: N-1) * t negated exp]
                ifFalse: [^0.0]! !

!Gamma methodsFor: 'private' stamp: ''!
gamma: n
        | t |
        t := n - 1.0.
        ^self computeSample: t outOf: t! !

!Gamma methodsFor: 'accessing' stamp: ''!
mean
        ^super mean*N! !

!Gamma methodsFor: 'private' stamp: ''!
setEvents: events
        N := events! !

!Gamma methodsFor: 'accessing' stamp: ''!
variance
        ^super variance*N! !

ContinuousProbability subclass: #Normal
	instanceVariableNames: 'mu sigma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Normal commentStamp: '<historical>' prior: 0!
How long before a success occurs or how many events occur in a certain time interval?!

!Normal class methodsFor: 'instance creation' stamp: ''!
mean: a deviation: b
        b > 0.0
        ifTrue: [^self new setMean: a standardDeviation: b]
        ifFalse: [self error: 'standard deviation must be greater than 0.0']! !

!Normal methodsFor: 'probability functions' stamp: ''!
density: x

        | twoPi t |
        twoPi := 2 * 3.1415926536.
        t := x - mu/sigma.
        ^(-0.5 * t squared) exp / (sigma * twoPi sqrt)! !

!Normal methodsFor: 'accessing' stamp: ''!
mean
        ^mu! !

!Normal methodsFor: 'random sampling' stamp: ''!
next
        | v1 v2 s rand u |
        rand := Uniform from: -1.0 to: 1.0.
        [v1 := rand next.
         v2 := rand next.
         s := v1 squared + v2 squared.
         s >= 1] whileTrue.
        u := (-2.0 * s ln /s) sqrt.
        ^mu + (sigma * v1 *u)! !

!Normal methodsFor: 'private' stamp: ''!
setMean: m standardDeviation: s
        mu := m.
        sigma := s! !

!Normal methodsFor: 'accessing' stamp: ''!
variance
        ^sigma squared! !

ContinuousProbability subclass: #Uniform
	instanceVariableNames: 'startNumber stopNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Uniform class methodsFor: 'instance creation' stamp: ''!
from: begin to: end  
        begin > end
                ifTrue: [self error: 'illegal interval']
                ifFalse: [^self new setStart: begin toEnd: end]! !

!Uniform methodsFor: 'probability functions' stamp: ''!
density: x  
        (x between: startNumber and: stopNumber)
                ifTrue: [^1.0 / (stopNumber - startNumber)]
                ifFalse: [^0]! !

!Uniform methodsFor: 'private' stamp: ''!
inverseDistribution: x
        "x is a random  number between 0 and 1"
        ^startNumber + (x * (stopNumber - startNumber))! !

!Uniform methodsFor: 'accessing' stamp: ''!
mean
        ^ (startNumber + stopNumber)/2! !

!Uniform methodsFor: 'private' stamp: ''!
setStart: begin toEnd: end
        startNumber := begin.
        stopNumber := end! !

!Uniform methodsFor: 'accessing' stamp: ''!
variance
        ^ (stopNumber - stopNumber) squared / 12! !

ProbabilityDistribution subclass: #DiscreteProbability
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

DiscreteProbability subclass: #Bernoulli
	instanceVariableNames: 'prob'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Bernoulli commentStamp: '<historical>' prior: 0!
Does an event occur?

density function answers the probability of occurrence of one of two events!

!Bernoulli class methodsFor: 'Examples' stamp: ''!
Cardgame

        "is the first draw of a card an ace?"
        (Bernoulli parameter: 4/52) next

        "does a car arrive in the next second?"

        "will a machine break down today?"! !

!Bernoulli class methodsFor: 'instance creation' stamp: ''!
parameter: aNumber
        (aNumber between: 0.0 and: 1.0)
                ifTrue: [^self new setParameter: aNumber]
                ifFalse: [^self error: 'The probability must be between 0.0 and 1.0']! !

!Bernoulli methodsFor: 'probability functions' stamp: ''!
density: x

        x=1 ifTrue: [^prob].
        x=0 ifTrue: [^1.0-prob].
        self error: ' outcomes of a Bernoulli can only be 1 or 0'! !

!Bernoulli methodsFor: 'private' stamp: ''!
inverseDistribution: x

        x <= prob
                ifTrue: [^1]
                ifFalse: [^0]! !

!Bernoulli methodsFor: 'accessing' stamp: ''!
mean
        ^prob! !

!Bernoulli methodsFor: 'private' stamp: ''!
setParameter: aNumber
        prob :=aNumber! !

!Bernoulli methodsFor: 'accessing' stamp: ''!
variance
        ^prob * (1.0 - prob)! !

Bernoulli subclass: #Binomial
	instanceVariableNames: 'N'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Binomial commentStamp: '<historical>' prior: 0!
how many successes occurred in N trials?

density funciton answers what is the probability that x successes will occur in the next N trials?

i.e., N repeated Bernoulli trials!

!Binomial class methodsFor: 'Examples' stamp: ''!
FlippingCoins

        | sampleA sampleB |
        sampleA := Bernoulli parameter: 0.5.

        "Did I get heads?"
        sampleA next.

        sampleB := Binomial events: 5 mean: 2.5.
         
        "How many heads did I get in 5 trials?"
        sampleB next! !

!Binomial class methodsFor: 'instance creation' stamp: ''!
events: n mean: m

        n truncated <= 0 ifTrue: [self error: 'number of events must be > 0'].
        ^self new events: n mean: m! !

!Binomial methodsFor: 'probability functions' stamp: ''!
density: x

        (x between: 0 and: N)
                ifTrue: [^((self computeSample: x outOf: N) / (self computeSample: x outOf: x)) * (prob raisedTo: x)*((1.0 - prob) raisedTo: N-x)]
                ifFalse: [^0.0]! !

!Binomial methodsFor: 'private' stamp: ''!
events: n mean: m

        N := n truncated.
        self setParameter: m/N! !

!Binomial methodsFor: 'random sampling' stamp: ''!
next
         
        |t|
        t := 0.
        N timesRepeat: [t := t + super next].
        ^t! !

Bernoulli subclass: #Geometric
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Geometric commentStamp: '<historical>' prior: 0!
How many repeated, independent Bernoulli trials are needed before the first success is obtained?

e.g., how many seconds before the next car arrives (as versus how many cars arrive in the next 20 sec as in a binomial question)!

!Geometric class methodsFor: 'Examples' stamp: ''!
CarsArriving

        | sample |
         
        "two cars arrive every minute"
        sample := Geometric mean: 60/2.

        "what is the probability that it will take 30 sec before the next car arrives?"
        sample density: 30.

        "Did the next car arrive in 30 to 40 seconds?"
        sample distribution: (30 to: 40)! !

!Geometric class methodsFor: 'instance creation' stamp: ''!
mean: m

        ^self parameter: m! !

!Geometric methodsFor: 'probability functions' stamp: ''!
density: x

        x>0 ifTrue: [^prob * ((1.0 - prob) raisedTo: x-1)]
                ifFalse: [^0.0]! !

!Geometric methodsFor: 'private' stamp: ''!
inverseDistribution: x

        ^(x ln / (1.0 - prob) ln) ceiling! !

!Geometric methodsFor: 'accessing' stamp: ''!
mean
        ^ 1.0 / prob! !

!Geometric methodsFor: 'accessing' stamp: ''!
variance
        ^ (1.0 - prob) / prob squared! !

!DiscreteProbability methodsFor: 'probability functions' stamp: ''!
distribution: aCollection
        "Answer the sum of the discrete values of the density function for each element in the collection."

        | t |
        t := 0.0.
        aCollection do: [:i | t := t + (self density: i)].
        ^t! !

DiscreteProbability subclass: #Poisson
	instanceVariableNames: 'mu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Poisson commentStamp: '<historical>' prior: 0!
how many events occur in a unit time?
        used for sampling potential demands by customers for service

        The Poisson is typically the rate at which the service is provided.

        density function determines the probability that, in a unit interval, x events will occur.
!

!Poisson class methodsFor: 'instance creation' stamp: ''!
mean: p

        p > 0.0
                ifTrue: [^self new setMean: p]
                ifFalse: [self error: 'mean must be greater than 0.0']! !

!Poisson methodsFor: 'probability functions' stamp: ''!
density: x

        x >= 0
                ifTrue: [^ ((mu raisedTo: x) * (mu negated exp)) / x factorial]
                ifFalse: [^0.0]! !

!Poisson methodsFor: 'accessing' stamp: ''!
mean
        ^mu! !

!Poisson methodsFor: 'random sampling' stamp: ''!
next
        | p n q |
        p := mu negated exp.
        n := 0.
        q := 1.0.
        [q := q * U next.
         q >= p]
                whileTrue: [n := n + 1].
        ^n! !

!Poisson methodsFor: 'private' stamp: ''!
setMean: p
        mu := p! !

!Poisson methodsFor: 'accessing' stamp: ''!
variance
        ^mu! !

DiscreteProbability subclass: #SampleSpace
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!SampleSpace class methodsFor: 'instance creation' stamp: ''!
data: aCollection
         
        ^self new setData: aCollection! !

!SampleSpace class methodsFor: 'Examples' stamp: ''!
heights

        | heights |
        heights := SampleSpace data: #(60 60 60 62 62 64 64 64 64 66 66 66 68 68 68 68 68 70 70 70).
         
        "what is the probability of randomly selecting a student with height 64?"
        heights density: 64.

        "what is the probability of randomly selecting a student whose height is between 60 and 64?"
        heights distribution: (60 to: 64 by: 2)! !

!SampleSpace methodsFor: 'probability functions' stamp: ''!
density: x
        "x must be in the sample space;  the probability must sum over all occurrences of x in the sample space."

        (data includes: x)
                ifTrue: [^(data occurrencesOf: x) / data size]
                ifFalse: [^0]! !

!SampleSpace methodsFor: 'private' stamp: ''!
inverseDistribution: x
         
        ^data at: (x*data size) truncated + 1! !

!SampleSpace methodsFor: 'private' stamp: ''!
setData: aCollection
         
        data := aCollection! !

!ProbabilityDistribution class methodsFor: 'class initialization' stamp: ''!
initialize
        "Uniformly distributed random numbers in the range [o,1]."

        U := Random new! !

!ProbabilityDistribution class methodsFor: 'instance creation' stamp: ''!
new
         
        ^self basicNew! !

!ProbabilityDistribution methodsFor: 'random sampling' stamp: ''!
atEnd
        ^false! !

!ProbabilityDistribution methodsFor: 'private' stamp: ''!
computeSample: m outOf: n

        m>n ifTrue: [^0.0].
        ^n factorial / (n-m) factorial! !

!ProbabilityDistribution methodsFor: 'probability functions' stamp: ''!
density: x

        self subclassResponsibility! !

!ProbabilityDistribution methodsFor: 'probability functions' stamp: ''!
distribution: aCollection

        self subclassResponsibility! !

!ProbabilityDistribution methodsFor: 'private' stamp: ''!
inverseDistribution: x
        self subclassResponsibility! !

!ProbabilityDistribution methodsFor: 'random sampling' stamp: ''!
next
        "This is a general random number generation method for any probability law;  use the (0,1) uniformly distributed random varible U as the value of the law's distribution function.  Obtain the next random value and then solve for the inverse.  The inverse solution is defined by the subclass."

        ^self inverseDistribution: U next! !

RTHTML5Exporter subclass: #MassRTHTML5Exporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

!MassRTHTML5Exporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/3/2016 16:42'!
addEnd
	self writeText:
'}
		</script>'.! !

!MassRTHTML5Exporter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/3/2016 16:45'!
addStart: aView
	self writeText: 
'<script type="text/javascript">
			var protocol = ("https:" == document.location.protocol)? "https" : "http";
			document.write(unescape("%3Cscript src=\""+ protocol + "://code.jquery.com/jquery-1.11.0.min.js\" type=\"text/javascript\"%3E%3C/script%3E"));
			document.write(unescape("%3Cscript src=\""+ protocol + "://code.jquery.com/jquery-migrate-1.2.1.min.js\" type=\"text/javascript\"%3E%3C/script%3E"));
		</script>
		<script src="',self roassalJSfile,'"></script>
		<style type="text/css">
			body, svg { 
				margin: 0;
				padding: 0;
				-moz-user-select:-moz-none;
				-moz-user-select: none;
				-o-user-select: none;
				-khtml-user-select:none;
				-webkit-user-select:none;
				-ms-user-select:none;
				user-select:none;
			}
			text{
				cursor: default;
			}
		</style>
		<script type="text/javascript">
function buildView() {
	'. ! !

Object subclass: #DelayedEvent
	instanceVariableNames: 'resumptionSemaphore resumptionCondition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!DelayedEvent class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!DelayedEvent class methodsFor: 'instance creation' stamp: ''!
onCondition: anObject
	^super new setCondition: anObject! !

!DelayedEvent methodsFor: 'comparing' stamp: ''!
<= aDelayedEvent
	"Answer whether the receiver should be sequenced before the argument."
	
	resumptionCondition isNil
		ifTrue: [^true]
		ifFalse: [^resumptionCondition <= aDelayedEvent condition]! !

!DelayedEvent methodsFor: 'accessing' stamp: ''!
condition
	"Answer a condition under which the event should be sequenced."

	^resumptionCondition! !

!DelayedEvent methodsFor: 'accessing' stamp: ''!
condition: anObject
	"The argument, anObject, is the condition under which the event should be sequenced."

	resumptionCondition := anObject! !

!DelayedEvent methodsFor: 'private' stamp: ''!
initialize
	resumptionSemaphore := Semaphore new! !

!DelayedEvent methodsFor: 'scheduling' stamp: ''!
pause
	"Suspend the current active process, that is, the current event that is running."

	Simulation active stopProcess.
	resumptionSemaphore wait! !

!DelayedEvent methodsFor: 'scheduling' stamp: ''!
resume
	"Resume the suspended process."
	Simulation active startProcess.
	resumptionSemaphore signal.
	^resumptionCondition! !

!DelayedEvent methodsFor: 'private' stamp: ''!
setCondition: anObject
	self initialize.
	resumptionCondition := anObject! !

DelayedEvent subclass: #WaitingSimulationObject
	instanceVariableNames: 'amount resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!WaitingSimulationObject class methodsFor: 'instance creation' stamp: ''!
for: amount of: aResource withPriority: aNumber
	^(self onCondition: aNumber) setAmount: amount resource: aResource! !

!WaitingSimulationObject class methodsFor: 'instance creation' stamp: ''!
for: amount withPriority: aNumber
	^(self onCondition: aNumber) setAmount: amount resource: nil! !

!WaitingSimulationObject methodsFor: 'accessing' stamp: ''!
amount
	^amount! !

!WaitingSimulationObject methodsFor: 'task language' stamp: ''!
consume: aNumber
	amount := (amount - aNumber) max: 0! !

!WaitingSimulationObject methodsFor: 'accessing' stamp: ''!
name
	^resource name! !

!WaitingSimulationObject methodsFor: 'task language' stamp: ''!
release
	resource produce: amount.
	amount := 0! !

!WaitingSimulationObject methodsFor: 'task language' stamp: ''!
release: anAmount
	resource produce: anAmount.
	amount := amount - anAmount! !

!WaitingSimulationObject methodsFor: 'accessing' stamp: ''!
resource
	^resource! !

!WaitingSimulationObject methodsFor: 'accessing' stamp: ''!
resource: aResource
	resource := aResource! !

!WaitingSimulationObject methodsFor: 'private' stamp: ''!
setAmount: aNumber resource: aResource
	amount := aNumber.
	resource := aResource! !

Object subclass: #Histogram
	instanceVariableNames: 'tallyArray lowerBound upperBound step minValue maxValue totalValues extraEntries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!Histogram class methodsFor: 'class initialization' stamp: ''!
from: lowerNum to: upperNum by: step
        ^self new newLower: lowerNum upper: upperNum by: step! !

!Histogram methodsFor: 'accessing' stamp: ''!
contains: aValue
        ^lowerBound <= aValue and: [aValue < upperBound]! !

!Histogram methodsFor: 'printing' stamp: ''!
firstHeader: aStream
        aStream cr; tab.
        aStream nextPutAll: 'Number of '.
        aStream tab.
        aStream nextPutAll: 'Minimum '.
        aStream tab.
        aStream nextPutAll: 'Maximum '.
        aStream tab.
        aStream nextPutAll: 'Average '.
        aStream cr; tab.
        aStream nextPutAll: 'Objects '.
        aStream tab.
        aStream nextPutAll: 'Value '.
        aStream tab.
        aStream nextPutAll: 'Value '.
        aStream tab.
        aStream nextPutAll: 'Value '.! !

!Histogram methodsFor: 'private' stamp: ''!
newLower: lowerNum upper: upperNum by: stepAmount
        tallyArray := Array new: (upperNum - lowerNum // stepAmount).
        tallyArray atAllPut: 0.
        lowerBound := lowerNum.
        upperBound := upperNum.
        step := stepAmount.
        minValue := maxValue := nil.
        totalValues := 0.
        extraEntries := 0! !

!Histogram methodsFor: 'printing' stamp: ''!
printStatisticsOn: aStream  
        | totalObjs pos |
        self firstHeader: aStream.
        aStream cr; tab.
        totalObjs := extraEntries.
        tallyArray do: [:each | totalObjs := totalObjs + each].
        totalObjs printOn: aStream.
        aStream tab.
        minValue printOn: aStream.
        aStream tab.
        maxValue printOn: aStream.
        aStream tab.
        (totalValues / totalObjs) asFloat printOn: aStream.
        aStream cr.
        self secondHeader: aStream.
        aStream cr.
        pos := lowerBound.
        tallyArray do:  
                [:entry |  
                pos printOn: aStream.
                aStream nextPut: $-.
                (pos := pos + step) printOn: aStream.
                aStream tab.
                entry printOn: aStream.
                aStream tab.
                (entry / totalObjs) asFloat printOn: aStream.
                aStream tab.
                aStream nextPut: $|.
                entry rounded timesRepeat: [aStream nextPut: $X].
                aStream cr]! !

!Histogram methodsFor: 'printing' stamp: ''!
secondHeader: aStream
        aStream cr; tab.
        aStream nextPutAll: 'Number of '.
        aStream cr.
        aStream nextPutAll: 'Entry '.
        aStream tab.
        aStream nextPutAll: 'Objects '.
        aStream tab.
        aStream nextPutAll: 'Frequency '.! !

!Histogram methodsFor: 'accessing' stamp: ''!
store: aValue  
        | index |
        minValue isNil
                ifTrue: [minValue := maxValue := aValue]
                ifFalse:  
                        [minValue := minValue min: aValue.
                        maxValue := maxValue max: aValue].
        totalValues := totalValues + aValue.
        (self contains: aValue)
                ifTrue:  
                        [index := (aValue - lowerBound // step) + 1.
                        tallyArray at: index put: (tallyArray at: index) + 1]
                ifFalse: [extraEntries := extraEntries + 1]! !

Object subclass: #MassBackEnd
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

MassBackEnd class
	instanceVariableNames: 'uniqueFileId'!

MassBackEnd class
	instanceVariableNames: 'uniqueFileId'!

!MassBackEnd class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:36'!
uniqueFileId

	uniqueFileId ifNil: [ uniqueFileId := 0].
	^ uniqueFileId := uniqueFileId + 1! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/3/2016 15:51'!
allMetrics: entity
	
	^ {#CFCRel -> entity metricCFCRel.
	#NOA -> entity metricNOA.
	#NOAC -> entity metricNOAC.
	#PDiffic -> entity metricProcessDifficulty.
	#PLength -> entity metricProcessLength.
	#PVolume -> entity metricProcessVolume} asDictionary 
	
	
	
	
	! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/2/2016 12:25'!
assessement: request

	^ true! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:31'!
dynAnalysis01For: simulation
		|view out model cv values eb stream |

	

	out :=  simulation out.
	view := RTMondrian new.
	model := simulation processModel.
	"Execute the visulization"
	
	cv := A4BPUIBVFactory color.
	view interaction action:[ :n | n inspect].
	values := model flowElements.
	view shape rectangle color:[: n | (n accept: cv)].
	view shape rectangle 
		width:[: n | (out getParameterAt: n) processing vv ];
		height:[: n | ( n metricCFC  + 1 ) ];
		color:[: n | (n accept: cv) alpha: 0.9 "1 - (1 / (n numberOfRelations + 1) )"].
	view nodes: values.
	view normalizer
	normalizeColor: #numberOfRelations "[:n | (out getParameterAt: n ) quantity vv] "using: (ColorPalette sequential colors: 9 scheme: 'YlOrRd') using: #value.
	eb := view edges.
	eb shape line color:(Color lightGray "alpha:0.2").
	eb connectFrom:#yourself toAll:[ :n | n outFlows].
	view layout tree.
	view build.
	
	stream := WriteStream on: String new.
	
	RTHTML5Exporter new 	
		export: view view toStream: stream.
		
	^ stream contents 
	! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:31'!
dynAnalysis02For: simulation
	
	| ds cv i b scenario cvv stream |
	" each simulation is a scenario to display"
	b := RTGrapher new.
	"b extent: 600 @ 300."
	b interaction popup.
	b view @ RTDraggableView.
	cv := Dictionary new.
	cvv := A4BPUIBVFactory color.
	i := 0.
	scenario :=  simulation fullOutputScenario.
	scenario elementParameters do: [ :p | 
		cv at: p elementRef id put: ( p  elementRef accept: cvv ) ].
	i := i + 1.
	ds := RTStackedDataSet new.
	ds interaction popup text: [ :el | el aName ].
	ds dotShape rectangle
		width:[ :el |  | p v |
			p := scenario getParameterAt: el.
			v := ( p quantity vv ) + 1. 
			v * 2];
		height: 5;
		color: [ :el | cv at: el id ].
	ds points: simulation processModel allActivities.
	ds y: [ :el |  | p v |
			p := scenario getParameterAt: el.
			v := ( p processing vv ) + 1. 
			v ].
	ds x: [ :el | i := i + 1 ].
	b add: ds.
	b axisX title: 'Activities'.
	b axisY title: 'Processing time'.
	b build.
	
	stream := WriteStream on: String new.
	
	RTHTML5Exporter new 	
		export: b view toStream: stream.
		
	^ stream contents 
	! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:45'!
htmlResultsAsUrl: results
	
	|fileName|
	
	fileName := '/assets/generatedHtml/' , self class uniqueFileId asString , '.htm'.
	
	(FileStream forceNewFileNamed: 
		MassServer filePath , fileName)
		nextPutAll: results;
		close.
		
	^ fileName 
	
	
	
	! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 16:10'!
open: request
	
	"should return list of all available plans + photo, description"
	
	^ true! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 16:05'!
openFile: request

	|file contents|

	file := FileStream fileNamed: 
		MassServer filePath , '/assets/bpmn/' , (request at: #'eventSpecific[filename]') , '.bpmn'.
	contents := file contents.
	Transcript show: contents.
	file close.
	
	^ contents! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/2/2016 12:17'!
processModelFor: planFile
	
	|stream xmlContents parser processModel|
	
	stream := FileStream fileNamed: 
		MassServer filePath , '/assets/bpmn/' , planFile , '.bpmn'.
	xmlContents := stream contents.
	stream close.
	parser := BPMN20Parser parseString: xmlContents.
	^ processModel := parser definitions first.! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 16:06'!
runAssessement: request

	^ true! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 16:05'!
save: request

	|file newContents|

	file := FileStream fileNamed: 
		MassServer filePath , '/assets/bpmn/' , (request at: #'planFile') , '.bpmn'.
	newContents := request at: #'eventSpecific'.
	file truncate; nextPutAll: newContents; flush; close.
	
	^ true! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:42'!
saveRunScenario: request

 	| scenarioValues tasks scenario simb gateways processModel allMetrics staticAnalysis simulation htmlForDynAnalysis01 htmlForDynAnalysis02 |
	
	scenarioValues := request at: #'currentScenario[]'.
	tasks := self tasksFor: (request at: #'planFile').
	scenario := Dictionary new.
	tasks doWithIndex: [:t :index |  scenario at: t put: (scenarioValues at: index)	].
		
	processModel := self processModelFor: (request at: #'planFile').	
	simb := BPSimBuilder buildScenario.
	simb processModel: processModel.

	scenario keysAndValuesDo: [ :task :value | 
		simb timer 
			processingTime: ( simb duration value: value ) inElementName: task.
	].
	
	gateways := (processModel activities select: [ :a | a class superclass = BPMN20GatewayImpl ]).

	gateways do:[ :g | simb control probabilityParameterOnGateway: g ].

	scenario :=  simb currentScenario.
	
	
	allMetrics := self allMetrics: processModel.
	staticAnalysis := self htmlResultsAsUrl: (self staticAnalysisFor: processModel).
	simulation := self simulate: scenario for: processModel.
	htmlForDynAnalysis01 := self htmlResultsAsUrl: (self dynAnalysis01For: simulation).
	htmlForDynAnalysis02 := self htmlResultsAsUrl: (self dynAnalysis02For: simulation).
	
	"this should be decomposed and provide raw metrics for time, activities, flow-elements metrics etc which will then be assessed"

	
	^ { allMetrics . staticAnalysis .  htmlForDynAnalysis01 . htmlForDynAnalysis02 }
	
	
	
	
	! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/2/2016 11:47'!
scenarioForm: request
	^ self tasksFor: (request at: #'planFile')! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/3/2016 15:03'!
simulate: scenario for: processModel

	|bpSimEngine bpSimExecution simProcesses seconds|
	seconds := 0.
	bpSimEngine := BPSimBPMN20EngineProcess new.
	bpSimEngine addProcessModel: processModel.
	simProcesses := bpSimEngine runSimulationUsing: scenario for: 1.
	[ (simProcesses select: [ :sp | sp isTerminated not ]) isEmpty ]
		whileFalse: [ (Delay forSeconds: 1) wait. seconds := seconds + 1. 
			Transcript 
				show: '===========';cr;
				show: seconds; show: ' sec'; cr;
				show: '===========';cr].
			
	^ bpSimExecution := bpSimEngine simulations first.! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/11/2016 16:31'!
staticAnalysisFor: processModel 
	
	|rtView view model cv eb lb values entities stream |
	rtView := RTView new.
	view := RTMondrian new.
	view view: rtView.
	rtView @ RTDraggableView .
	model := processModel.
	
	"Execute the visulization"
	
	cv := A4BPUIBVFactory color.
	view interaction action:[ :n | n inspect].
	values := model flowElements.
	view shape rectangle 
		width:[: nn | (nn metricCFC + 1) * 10 ];
		height:[: nn | ( nn metricCFCAbs  + 1 ) * 10 ];
		color:[: nn | (nn accept: cv) alpha:  0.9].
	view nodes: values.
	"Blues, PuBuGn, BuPu, GnBu, YlGn, Greens, YlOrBr, YlOrRd, Greys, Reds, YlGnBu, PuRd, RdPu, PuBu, BuGn, Oranges, Purples, OrRd"
	view normalizer
	normalizeColor: #numberOfRelations using: (ColorPalette sequential colors: 9 scheme: 'YlOrRd') using: #value.
	eb := view edges.
	eb shape line color:(Color lightGray).
	eb connectFrom:#yourself toAll:[ :n | n outFlows].
	view layout tree.
	view build.
	
	
	lb := RTLegendBuilder new view: view view.
	lb addText:'width:[: n | n metricCFCAbs]'.
	lb addText:'height:[: n | n metricCFC]'.
	lb addText:'color alpha:[: n | n numberOfRelations]'.
	entities := Dictionary new.
	values do:[ :p | entities at: p class put: p class].
	entities do:[ :p |  lb addColor:(p new accept: cv) text: p bpmn20Name ].
	lb right.
	lb build.
	
	
	stream := WriteStream on: String new.
	
	RTHTML5Exporter new 	
		export: view view toStream: stream.
		
	^ stream contents ! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/2/2016 12:16'!
tasksFor: planFile
	|processModel|
	processModel := self processModelFor: planFile.
	^ (processModel activities select: [ :a | a class = BPMN20TaskImpl  ])
		collect: [ :task | task name ]! !

!MassBackEnd methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 16:06'!
viewEdit: request

	^ true! !

Object subclass: #MassCausalConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassCausalConnection subclass: #ASTCausalConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassCausalConnection subclass: #BPMNCausalConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassCausalConnection subclass: #ObjGraphCausalConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassCausalConnection subclass: #TraceCausalConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

Object subclass: #MassLog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassMetric
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassOption
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassPlan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassProject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #AST2BPMNProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #AST2TraceProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #BPMN2ASTProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #BPMN2ObjGraphProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #BPMN2TraceProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #ObjGraph2BPMNProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #Trace2ASTProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjection subclass: #Trace2BPMNProjection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

Object subclass: #MassProjectionModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjectionModel subclass: #MassASTModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjectionModel subclass: #MassBPMNModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjectionModel subclass: #MassObjGraphModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

MassProjectionModel subclass: #MassTraceModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Projection'!

Object subclass: #MassScenario
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassServer
	instanceVariableNames: 'mBackEnd teapotInstance mode allowedRequests'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

!MassServer class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/29/2016 17:58'!
filePath

^ SmalltalkImage current 
	imageDirectory parent parent 
	parent asFileReference pathString , '/static/'! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/3/2016 16:33'!
allowedRequests

	^ allowedRequests ifNil:[ allowedRequests := MassBackEnd selectors ].! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/3/2016 16:50'!
deployment

	teapotInstance := Teapot 
		configure: { 
			#defaultOutput -> #json. 
			#port -> 8080. 
			#debugMode -> false }.

	self serveStatic.
	
	teapotInstance "this is for development ONLY for obvious reasons"
		POST: '/api/<expression>' -> [ :req | self deploymentResponse: req ].! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/3/2016 16:38'!
deploymentResponse: req

	^ [self sanitize: req. mBackEnd perform: (req at: #expression) asSymbol with: req] 
				on: Error 
				do: [ :ex | ex description ].
		"it expects expressions that return compatible values for JSON encoding"
			! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/3/2016 16:18'!
development

	teapotInstance := Teapot 
		configure: { 
			#defaultOutput -> #json. 
			#port -> 8080. 
			#debugMode -> true }.
			
	self serveStatic.
	
	teapotInstance "this is for development ONLY for obvious reasons"
		POST: '/api/<expression>' -> [ :req | self developmentResponse: req ].
			! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/3/2016 16:37'!
developmentResponse: req

	^ [self sanitize: req. mBackEnd perform: (req at: #expression) asSymbol with: req] 
				on: Error 
				do: [ :ex | ex description ].
		"it expects expressions that return compatible values for JSON encoding"
			! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/25/2016 18:01'!
forDeployment

	mode := #deployment.

	"teapotInstance := Teapot 
		configure: { 
			#defaultOutput -> #json. 
			#port -> 8080 }.	"! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/25/2016 18:01'!
forDevelopment

	mode := #development! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 3/21/2016 15:45'!
initialize

	mBackEnd := MassBackEnd new.
	allowedRequests := nil
	! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 3/21/2016 15:42'!
sanitize: req

	(self allowedRequests includes: (req at: #expression))
		ifFalse: [ Error signal: 'Not a valid Request: ' , (req at: #expression) ]
	! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/29/2016 17:57'!
serveStatic

	teapotInstance 
		serveStatic: '/' 
		from: self class filePath.
	! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/25/2016 18:01'!
start

	self perform: mode.
	teapotInstance start.
	! !

!MassServer methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/25/2016 17:38'!
stop

	teapotInstance start
	! !

Object subclass: #MassSimulation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #MassVisualization
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

Object subclass: #Resource
	instanceVariableNames: 'pending resourceName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!Resource class methodsFor: 'instance creation' stamp: ''!
named: resourceName
	^self new setName: resourceName! !

!Resource methodsFor: 'accessing' stamp: ''!
acquire
	^self! !

!Resource methodsFor: 'accessing' stamp: ''!
addRequest: aDelayedEvent
	pending add: aDelayedEvent.
	self provideResources.
	aDelayedEvent pause.! !

!Resource methodsFor: 'accessing' stamp: ''!
name
	^resourceName! !

!Resource methodsFor: 'private' stamp: ''!
provideResources
	^self! !

!Resource methodsFor: 'private' stamp: ''!
setName: aString
	resourceName := aString.
	pending := SortedCollection new! !

Resource subclass: #ResourceCoordinator
	instanceVariableNames: 'whoIsWaiting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!ResourceCoordinator methodsFor: 'task language' stamp: ''!
acquire
	|waiting|
	self customersWaiting ifTrue: [^self giveService].
	"get here if there is no customer waiting for the server."
	waiting := WaitingSimulationObject for: 1 withPriority: 0.
	whoIsWaiting := #server.
	self addRequest: waiting.
	^waiting resource! !

!ResourceCoordinator methodsFor: 'accessing' stamp: ''!
customersWaiting
	^whoIsWaiting == #customer! !

!ResourceCoordinator methodsFor: 'private' stamp: ''!
getServiceFor: aCustomerRequest

	| aServerRequest |
	aServerRequest := pending removeFirst.
	pending isEmpty ifTrue: [whoIsWaiting := #none].
	aServerRequest resource: aCustomerRequest.
	aServerRequest resume.
	aCustomerRequest pause! !

!ResourceCoordinator methodsFor: 'private' stamp: ''!
giveService
	|aCustomerRequest|
	aCustomerRequest:= pending removeFirst.
	pending isEmpty ifTrue: [whoIsWaiting := #none].
	^aCustomerRequest! !

!ResourceCoordinator methodsFor: 'task language' stamp: ''!
producedBy: aCustomer
	|waiting|
	waiting := WaitingSimulationObject for: 1
					of: aCustomer
					withPriority: 0.
	self serversWaiting ifTrue: [^self getServiceFor: waiting].
	whoIsWaiting := #customer.
	self addRequest: waiting! !

!ResourceCoordinator methodsFor: 'accessing' stamp: ''!
queueLength
	^pending size! !

!ResourceCoordinator methodsFor: 'accessing' stamp: ''!
serversWaiting
	^whoIsWaiting == #server! !

!ResourceCoordinator methodsFor: 'private' stamp: ''!
setName: aString
	super setName: aString.
	whoIsWaiting := #none! !

Resource subclass: #ResourceProvider
	instanceVariableNames: 'amountAvailable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!ResourceProvider class methodsFor: 'instance creation' stamp: ''!
named: aResourceName
	^self new setName: aResourceName with: 0! !

!ResourceProvider class methodsFor: 'instance creation' stamp: ''!
named: aResourceName with: amount
	^self new setName: aResourceName with: amount! !

!ResourceProvider methodsFor: 'task language' stamp: ''!
acquire: amountNeeded withPriority: priorityNumber 
	| waiting |
	waiting := WaitingSimulationObject
					for: amountNeeded 
					of: self
					withPriority: priorityNumber.
	self addRequest: waiting.
	^waiting! !

!ResourceProvider methodsFor: 'accessing' stamp: ''!
amountAvailable
	^amountAvailable! !

!ResourceProvider methodsFor: 'task language' stamp: ''!
produce: amount
	amountAvailable := amountAvailable + amount.
	self provideResources! !

!ResourceProvider methodsFor: 'private' stamp: ''!
provideResources
	| waiting |
	[pending isEmpty not and: [pending first amount <= amountAvailable]]
		whileTrue: 
			[waiting := pending removeFirst.
			amountAvailable := amountAvailable - waiting amount.
			waiting resume]! !

!ResourceProvider methodsFor: 'private' stamp: ''!
setName: aResourceName with: amount
	super setName: aResourceName.
	amountAvailable := amount! !

Object subclass: #SanityExample
	instanceVariableNames: 'parser processModel definition bpSimExecution bpSimEngine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass'!

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 14:24'!
allMetrics 
	
	^ { #CFCRel -> processModel metricCFCRel.
	#NOA -> processModel metricNOA .
	#NOAC -> processModel metricNOAC .
	#PDiffic -> processModel metricProcessDifficulty .
	#PLength -> processModel metricProcessLength .
	#PVolume -> processModel metricProcessVolume. }! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/16/2016 12:37'!
axisAdaptedExample

	"see in examples browser and replicate from there"! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 14:26'!
display
	
	self allMetrics inspect.
	self staticAnalysis inspect.
	self dynamicAnalysis inspect.
	! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 14:25'!
dynamicAnalysis 
	
	| view composer|
	view := RTView new.
	composer := RTComposer new.
	composer view:view.
	composer gap: 50.

	self plot01Basic: composer group:#BPMN.
	self plot01Historical: composer group: #Time.	


	"self plot01Mondrian: composer  group: #Mondrian01."
	
	composer move: #Time onTheRightOf: #BPMN.
	"composer move: #BPMN below: #Mondrian01."
	composer nameGroup: #BPMN as: 'Bussines Process Notation'.
	composer nameGroup: #Time as: 'Time information'.
	"composer nameGroup: #Mondrian01 as: 'Mondrian01'."
	composer view canvas camera focusOnCenterScaled.

	^ composer view! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 17:57'!
initialize 
	|stream xmlContents|
	stream := FileStream readOnlyFileNamed: 'ResponseTsunamiHM.bpmn'.
	xmlContents := stream contents.
	stream close.
	parser := BPMN20Parser parseString: xmlContents.
	processModel := parser definitions first.
	self simulate.
	! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/16/2016 12:31'!
plot01Basic: composer group: group
	|view out model cv values eb|

	out :=  bpSimExecution out.
	view := RTMondrian new.
	view view: composer view.
	model := bpSimExecution processModel.
	out := bpSimExecution out.	
	"Execute the visulization"
	
	cv := A4BPUIBVFactory color.
	view interaction action:[ :n | n inspect].
	values := model flowElements.
	view shape rectangle color:[: n | (n accept: cv)].
	view shape rectangle 
		width:[: n | (out getParameterAt: n) processing vv ];
		height:[: n | ( n metricCFC  + 1 ) ];
		color:[: n | (n accept: cv) alpha: 0.9 "1 - (1 / (n numberOfRelations + 1) )"].
	view nodes: values.
	view normalizer
	normalizeColor: #numberOfRelations "[:n | (out getParameterAt: n ) quantity vv] "using: (ColorPalette sequential colors: 9 scheme: 'YlOrRd') using: #value.
	eb := view edges.
	eb shape line color:(Color lightGray "alpha:0.2").
	eb connectFrom:#yourself toAll:[ :n | n outFlows].
	view layout tree.
	view build.
	
	
	
	
	composer layout: RTVerticalLineLayout new.
	composer propagateHighlight.
	composer group: group.
	composer propagateHighlightToAll.	! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 12:00'!
plot01Historical: composer group: group
	| ds cv i b scenario cvv|
	" each simulation is a scenario to display"
	b := RTGrapher new.
	"b extent: 600 @ 300."
	b view: composer view.
	b interaction popup.
	b view @ RTDraggableView.
	cv := Dictionary new.
	cvv := A4BPUIBVFactory color.
	i := 0.
	scenario :=  bpSimExecution fullOutputScenario.
	scenario elementParameters do: [ :p | 
		cv at: p elementRef id put: ( p  elementRef accept: cvv ) ].
	i := i + 1.
	ds := RTStackedDataSet new.
	ds interaction popup text: [ :el | el aName ].
	ds dotShape rectangle
		width:[ :el |  | p v |
			p := scenario getParameterAt: el.
			v := ( p quantity vv ) + 1. 
			v * 2];
		height: 5;
		color: [ :el | cv at: el id ].
	ds points: bpSimExecution processModel allActivities.
	ds y: [ :el |  | p v |
			p := scenario getParameterAt: el.
			v := ( p processing vv ) + 1. 
			v ].
	ds x: [ :el | i := i + 1 ].
	b add: ds.
	b axisX title: 'Activities'.
	b axisY title: 'Processing time'.
	b build.
	composer propagateHighlight.
	composer group: group! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/1/2016 15:58'!
scenario01
|simb task gateways ts|
simb := BPSimBuilder buildScenarioId: 'TsunamiScenario01'.
task := (processModel activities select: [ :a | a class = BPMN20TaskImpl  ]).

simb processModel: processModel.
ts := 5.

simb timer processingTime: ( simb duration value: 10 ) inElementName: 'T1 : Detect tsunami risk'.
simb timer processingTime: ( simb duration value: 15 ) inElementName: 'T2 : Inform tsunami start'.
simb timer processingTime: ( simb duration value: 10 ) inElementName:'T3 : Receive tsunami start '.
simb timer processingTime: ( simb duration value: 30 ) inElementName:'T4 : Fire the flares'.
simb timer processingTime: ( simb duration value: 30 ) inElementName: 'T5 : Notify functional units to start'.
simb timer processingTime: ( simb duration value: 60 ) inElementName: 'T6 : Inform by portable speakers'.
simb timer processingTime: ( simb duration value: 30 ) inElementName:'T7 : Broadcast over the media'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T8 : Evacuate people'.
simb timer processingTime: ( simb duration value: 180 ) inElementName: 'T8'' : Evacuate people'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T9 : Transfer injuries to safe places'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T9'' : Transfer injuries to safe places'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T10 : Inform, guide the fisherman'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T11 : Protect the property'.
simb timer processingTime: ( simb duration value: 180 ) inElementName:'T12 : Perform the first aid'.
simb timer processingTime: ( simb duration value: 30 ) inElementName:'T13 : Call ambulance '.
simb timer processingTime: ( simb duration value: 10 ) inElementName:'T14 : Detect tsunami end'.
simb timer processingTime: ( simb duration value: 15 ) inElementName:'T15 : Inform tsunami end'.
simb timer processingTime: ( simb duration value: 10 ) inElementName:'T16 : Receive tsunami end'.
simb timer processingTime: ( simb duration value: 30 ) inElementName:'T17 : Notify functional units to end'.
simb timer processingTime: ( simb duration value: 60 ) inElementName:'T18 : Identify damages'.
simb timer processingTime: ( simb duration value: 60 ) inElementName:'T18'' : Identify damages'.
simb timer processingTime: ( simb duration value: 60 ) inElementName:'T19 : Search distress fishermen'.
simb timer processingTime: ( simb duration value: 60 ) inElementName:'T19'' : Search distress fishermen'.
simb timer processingTime: ( simb duration value: 30 ) inElementName:'T20 : Close tsunami reponse'.



gateways := (processModel activities select: [ :a | a class superclass = BPMN20GatewayImpl ]).

gateways do:[ :g | simb control probabilityParameterOnGateway: g ].

^ simb currentScenario.! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/23/2016 12:46'!
simulate 
	|simProcesses seconds|
	seconds := 0.
	bpSimEngine := BPSimBPMN20EngineProcess new.
	bpSimEngine addProcessModel: processModel.
	simProcesses := bpSimEngine runSimulationUsing: self scenario01 for: 1.
	[ (simProcesses select: [ :sp | sp isTerminated not ]) isEmpty ]
		whileFalse: [ (Delay forSeconds: 1) wait. seconds := seconds + 1. 
			Transcript 
				show: '===========';cr;
				show: seconds; show: ' sec'; cr;
				show: '===========';cr].
	bpSimExecution := bpSimEngine simulations first.! !

!SanityExample methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 2/15/2016 18:06'!
staticAnalysis 
	
	|rtView view model cv eb lb values entities |
	rtView := RTView new.
	view := RTMondrian new.
	view view: rtView.
	rtView @ RTDraggableView .
	model := processModel.

	"Execute the visulization"
	
	cv := A4BPUIBVFactory color.
	view interaction action:[ :n | n inspect].
	values := model flowElements.
	view shape rectangle 
		width:[: nn | (nn metricCFC + 1) * 10 ];
		height:[: nn | ( nn metricCFCAbs  + 1 ) * 10 ];
		color:[: nn | (nn accept: cv) alpha:  0.9].
	view nodes: values.
	"Blues, PuBuGn, BuPu, GnBu, YlGn, Greens, YlOrBr, YlOrRd, Greys, Reds, YlGnBu, PuRd, RdPu, PuBu, BuGn, Oranges, Purples, OrRd"
	view normalizer
	normalizeColor: #numberOfRelations using: (ColorPalette sequential colors: 9 scheme: 'YlOrRd') using: #value.
	eb := view edges.
	eb shape line color:(Color lightGray).
	eb connectFrom:#yourself toAll:[ :n | n outFlows].
	view layout tree.
	view build.
	
	
	lb := RTLegendBuilder new view: view view.
	lb addText:'width:[: n | n metricCFCAbs]'.
	lb addText:'height:[: n | n metricCFC]'.
	lb addText:'color alpha:[: n | n numberOfRelations]'.
	entities := Dictionary new.
	values do:[ :p | entities at: p class put: p class].
	entities do:[ :p |  lb addColor:(p new accept: cv) text: p bpmn20Name ].
	lb right.
	lb build.
	
	^ view ! !

Object subclass: #Simulation
	instanceVariableNames: 'resources currentTime eventQueue processCount'
	classVariableNames: 'RunningSimulation'
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!Simulation commentStamp: '<historical>' prior: 0!
Expects to handle several kinds of resources, represented abstractly by class Resource, and concretely by subclasses of Resource:  ResourceProvider, StaticResource, and ResourceCoordinator!

Simulation subclass: #CarDealer
	instanceVariableNames: 'statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!CarDealer class methodsFor: 'demo' stamp: 'mjg 10/2/97 10:21'!
demo
	| aSimulation aStream|
	aSimulation := self new startUp.
	[aSimulation time < 180] whileTrue: [aSimulation proceed].
	aStream := FileStream fileNamed: 'dealer.events'.
	aSimulation printStatisticsOn: aStream.
	aStream close.! !

!CarDealer methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:50'!
defineArrivalSchedule

	self scheduleArrivalOf: CarBuyer 
		accordingTo: (Uniform from: 2 to: 6)
		startingAt: 1.0.

	self scheduleArrivalOf: (CarDelivery new) at: 90.0.

	"Only one delivery is scheduled; the instance of CarDelivery will reschedule itself."
! !

!CarDealer methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:47'!
defineResources

	self produce: 12 of: 'Car'.
! !

!CarDealer methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:50'!
exit: aSimulationObject

	super exit: aSimulationObject.

	"A CarDelivery could be exiting -- ignore it."

	(aSimulationObject isKindOf: CarBuyer)
		ifTrue: [statistics store: currentTime - aSimulationObject entryTime]
! !

!CarDealer methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:48'!
initialize

	super initialize.

	statistics := Histogram from: 1 to: 365 by: 7.
! !

!CarDealer methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:48'!
printStatisticsOn: aStream

	statistics printStatisticsOn: aStream.
! !

Simulation subclass: #Museum
	instanceVariableNames: 'statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!Museum class methodsFor: 'demo' stamp: 'mjg 10/2/97 10:19'!
demo
	| aSimulation aStream|
	aSimulation := self new startUp.
	[aSimulation time < 50] whileTrue: [aSimulation proceed].
	aStream := FileStream fileNamed: 'museum.events'.
	aSimulation printStatisticsOn: aStream.
	aStream close.! !

!Museum methodsFor: 'class initialization' stamp: ''!
defineArrivalSchedule
        self scheduleArrivalOf: Visitor accordingTo: (Uniform from: 5
to:10).! !

!Museum methodsFor: 'class initialization' stamp: ''!
exit: aSimulationObject
        super exit: aSimulationObject.
        statistics store: currentTime - aSimulationObject entryTime! !

!Museum methodsFor: 'class initialization' stamp: ''!
initialize
        super initialize.
        statistics := Histogram from: 5 to: 45 by: 5.! !

!Museum methodsFor: 'class initialization' stamp: ''!
printStatisticsOn: aStream
        statistics printStatisticsOn: aStream! !

Simulation subclass: #NothingAtAll
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!NothingAtAll class methodsFor: 'demos' stamp: 'vj 3/18/2004 23:17'!
aDoNothingDemo

        | aSimulation aFile |
        aFile := (FileStream newFileNamed: 'demo.events3').
        DoNothing file: aFile.
        aSimulation := self new startUp.
        [aSimulation time < 25] whileTrue: [aSimulation proceed].
        aSimulation finishUp. 	
        aFile close! !

!NothingAtAll methodsFor: 'initialization' stamp: 'MJG 7/28/97 11:18'!
defineArrivalSchedule
        self scheduleArrivalOf: DoNothing
                accordingTo: (Uniform from:1 to:5).! !

!Simulation class methodsFor: 'accessing' stamp: ''!
active
	^RunningSimulation! !

!Simulation class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!Simulation methodsFor: 'initialization' stamp: ''!
activate
	"This instance is now the active simulation"	
	RunningSimulation := self! !

!Simulation methodsFor: 'task language' stamp: ''!
coordinate: resourceName
	(self includesResourceFor: resourceName)
		ifFalse: [resources add: ( ResourceCoordinator named: resourceName)]! !

!Simulation methodsFor: 'initialization' stamp: ''!
defineArrivalSchedule
	"A subclass specifies the schedule by which simulation objects dynamically enter into the simulation."
	^self! !

!Simulation methodsFor: 'initialization' stamp: ''!
defineResources
	"A subclass specifies the schedule by which simulation objects that are initially entered into the simulation."
	^self! !

!Simulation methodsFor: 'scheduling' stamp: ''!
delayFor: timeDelay
	self delayUntil: currentTime+timeDelay! !

!Simulation methodsFor: 'scheduling' stamp: ''!
delayUntil: aTime
	| delayEvent |
	delayEvent := DelayedEvent onCondition: aTime.
	eventQueue add: delayEvent.
	"self stopProcess."
	delayEvent pause.
	"self startProcess"! !

!Simulation methodsFor: 'simulation control' stamp: ''!
enter: anObject
	^self! !

!Simulation methodsFor: 'simulation control' stamp: ''!
exit: anObject
	^self! !

!Simulation methodsFor: 'simulation control' stamp: 'vj 3/17/2004 18:21'!
finishUp
	[self  readyToContinue] whileFalse: [Processor yield].	"added by vj 3/17/2004 18:21"
	"We need to empty out the event queue."
	eventQueue := SortedCollection new.
	^nil! !

!Simulation methodsFor: 'accessing' stamp: ''!
includesResourceFor: resourceName 
	| test |
	test := resources 
				detect: [:each | each name = resourceName]
				ifNone: [nil].
	^test notNil! !

!Simulation methodsFor: 'initialization' stamp: ''!
initialize
	resources := Set new.
	currentTime := 0.0.
	processCount := 0.
	eventQueue := SortedCollection new! !

!Simulation methodsFor: 'scheduling' stamp: ''!
newProcessFor: aBlock
	self startProcess.
	[aBlock value. self stopProcess] fork! !

!Simulation methodsFor: 'simulation control' stamp: ''!
proceed
	| eventProcess |
	[self readyToContinue]
		whileFalse: [Processor yield].
	eventQueue isEmpty
		ifTrue: [^self finishUp]
		ifFalse: 
			[eventProcess := eventQueue removeFirst.
			currentTime := eventProcess condition.
			eventProcess resume]! !

!Simulation methodsFor: 'task language' stamp: ''!
produce: amount of: resourceName 
	(self includesResourceFor: resourceName)
		ifTrue: [(self provideResourceFor: resourceName)
				produce: amount
			"sends produce: to a ResourceProvider"]
		ifFalse: [resources add: (ResourceProvider named: resourceName with: amount)]! !

!Simulation methodsFor: 'accessing' stamp: ''!
provideResourceFor: resourceName
	^resources detect: [ :each | each name = resourceName]! !

!Simulation methodsFor: 'private' stamp: ''!
readyToContinue
	^processCount = 0! !

!Simulation methodsFor: 'reporting' stamp: ''!
report: label
	Transcript show: label.
	Transcript tab.
	Transcript show: 'pc: ', processCount printString.
	Transcript space.
	Transcript show: 'procs: ', (Processor processesAt: Processor userSchedulingPriority) printString.
	Transcript space.
	Transcript show: 'events: ', eventQueue size printString.
	Transcript cr! !

!Simulation methodsFor: 'task language' stamp: ''!
schedule: actionBlock after: timeDelay
	self schedule: actionBlock at: currentTime + timeDelay! !

!Simulation methodsFor: 'task language' stamp: ''!
schedule: aBlock at: timeInteger
	"This is the mechanism for scheduling a single action."
	self newProcessFor: [self delayUntil: timeInteger.  aBlock value]! !

!Simulation methodsFor: 'private' stamp: ''!
schedule: aBlock startingAt: timeInteger andThenEvery: aProbabilityDistribution 
	self newProcessFor: 
			[self delayUntil: timeInteger.
			self newProcessFor: aBlock copy.
			aProbabilityDistribution do: 
				[:nextTimeDelay | 
				self delayFor: nextTimeDelay.
				self newProcessFor: aBlock copy]]! !

!Simulation methodsFor: 'task language' stamp: ''!
scheduleArrivalOf: aSimulationObjectClass accordingTo: aProbabilityDistribution 
	"This means start now"

	self
		scheduleArrivalOf: aSimulationObjectClass
		accordingTo: aProbabilityDistribution
		startingAt: currentTime! !

!Simulation methodsFor: 'task language' stamp: ''!
scheduleArrivalOf: aSimulationObjectClass accordingTo: aProbabilityDistribution startingAt: timeInteger 
	"Note that aSimulationObjectClass is the class Simulation or one of 
	its subclasses.  The real work is done in the private message 
	schedule:startingAt:andThenEvery:. "

	self
		schedule: [aSimulationObjectClass new startUp]
		startingAt: timeInteger
		andThenEvery: aProbabilityDistribution! !

!Simulation methodsFor: 'task language' stamp: ''!
scheduleArrivalOf: aSimulationObject at: timeInteger 
	self schedule: [aSimulationObject startUp]
		at: timeInteger! !

!Simulation methodsFor: 'scheduling' stamp: ''!
startProcess
	processCount := processCount +1! !

!Simulation methodsFor: 'simulation control' stamp: ''!
startUp
	self activate.
	self defineResources.
	self defineArrivalSchedule! !

!Simulation methodsFor: 'scheduling' stamp: ''!
stopProcess
	processCount := processCount - 1! !

!Simulation methodsFor: 'accessing' stamp: ''!
time
	^currentTime! !

Simulation subclass: #StatisticsWithSimulation
	instanceVariableNames: 'statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!StatisticsWithSimulation methodsFor: 'simulation scheduling' stamp: ''!
enter: anObject  
        statistics at: anObject put: (SimulationObjectRecord new entrance: currentTime)! !

!StatisticsWithSimulation methodsFor: 'simulation scheduling' stamp: ''!
exit: anObject  
        (statistics at: anObject) exit: currentTime! !

!StatisticsWithSimulation methodsFor: 'initialization' stamp: ''!
initialize
        super initialize.
        statistics := Dictionary new.! !

!StatisticsWithSimulation methodsFor: 'statistics' stamp: ''!
printStatisticsOn: aStream  
        | stat |
        aStream cr.
        aStream nextPutAll: ' Object'.
        aStream tab.
        aStream nextPutAll: 'Entrance Time'.
        aStream tab.
        aStream nextPutAll: 'Duration'.
        aStream cr.
        stat := SortedCollection sortBlock: [:i :j | i value entrance <= j value entrance].
        statistics associationsDo: [:each | stat add: each].  
        stat do:  
                [:anAssociation |  
                aStream cr.
                anAssociation key printOn: aStream.
                aStream tab.
                anAssociation value printOn: aStream]! !

Simulation subclass: #Traffic
	instanceVariableNames: 'statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!Traffic methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:54'!
defineArrivalSchedule

	self scheduleArrivalOf: Car accordingTo: (Uniform from: 0.5 to: 2).
	self schedule: [self finishUp] at: 100.

	"MJG Note: DOESN'T WORK!!"
! !

!Traffic methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:52'!
initialize

	super initialize.
	statistics := Dictionary new.
	statistics at: #straight put: 0.
	statistics at: #right put: 0.
	statistics at: #left put: 0.
! !

!Traffic methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:54'!
printStatisticsOn: aStream

	aStream cr.
	aStream nextPutAll: 'Car Direction Tally'.
	statistics associationsDo: [:assoc | 
		aStream cr.
		assoc key printOn: aStream.
		aStream tab.
		assoc value printOn: aStream.]
! !

!Traffic methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:53'!
update: key

	statistics at: key put: (statistics at: key) + 1.
! !

Object subclass: #SimulationObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-DESimEngine'!

!SimulationObject commentStamp: '<historical>' prior: 0!
A SimulationObject represents any object that can be given a sequence of tasks to do.  The class specifies a general control sequence by which the object enters, carries out its tasks, and leaves the simulation.

	Resources can be produced, consumed, acquired, and tested by queries such as amountAvailable, serversWaiting, custoemrsWaiting?!

SimulationObject subclass: #Car
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!Car methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:51'!
tasks
	"Sample, without replacement, the direction through the intersection that the car will travel"

	| sample |

	sample := SampleSpace data: #(left left right straight straight straight straight straight).

	(Simulation active) update: sample next.
! !

SimulationObject subclass: #CarBuyer
	instanceVariableNames: 'entryTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!CarBuyer methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:46'!
entryTime

	^entryTime
! !

!CarBuyer methodsFor: 'initialization' stamp: 'vj 3/17/2004 16:55'!
initialize

	super initialize.
	entryTime := (Simulation active) time.
! !

!CarBuyer methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:47'!
tasks

	self acquire: 1 ofResource: 'Car'.
! !

SimulationObject subclass: #CarDelivery
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!CarDelivery methodsFor: 'simulation' stamp: 'vj 3/17/2004 16:54'!
tasks

	"Get access to the Car resource and produce 10, 11, or 12 cars."
	self produce: ((SampleSpace data: #(10 11 12)) next) ofResource: 'Car'.

	"Schedule a new delivery in 90 days"
	(Simulation active) scheduleArrivalOf: self at: (Simulation active) time + 90.! !

SimulationObject subclass: #EventMonitor
	instanceVariableNames: 'label'
	classVariableNames: 'Counter DataFile'
	poolDictionaries: ''
	category: 'Mass-Statistics'!

EventMonitor subclass: #DoNothing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!EventMonitor class methodsFor: 'class initialization' stamp: ''!
file: aFile
        DataFile := aFile.
        Counter := 0! !

!EventMonitor methodsFor: 'task language' stamp: ''!
acquire: amount ofResource: resourceName
        | aStaticResource |
        self timeStamp.
        DataFile nextPutAll: ' requests '.
        amount printOn: DataFile.
        DataFile nextPutAll: ' of ', resourceName.
        aStaticResource := super acquire: amount ofResource: resourceName.
        self timeStamp.
        DataFile nextPutAll: ' obtained '.
        amount printOn: DataFile.
        DataFile nextPutAll: ' of ', resourceName.
        ^aStaticResource! !

!EventMonitor methodsFor: 'task language' stamp: ''!
acquire: amount ofResource: resourceName withPriority: priorityNumber
        | aStaticResource |
        self timeStamp.
        DataFile nextPutAll: ' requests '.
        amount printOn: DataFile.
        DataFile nextPutAll: ' at priority '.
        priorityNumber printOn: DataFile.
        DataFile nextPutAll: ' of ', resourceName.
        aStaticResource := super acquire: amount ofResource: resourceName withPriority: priorityNumber.
        self timeStamp.
        DataFile nextPutAll: ' obtained '.
        amount printOn: DataFile.
        DataFile nextPutAll: ' of ', resourceName.
        ^aStaticResource! !

!EventMonitor methodsFor: 'task language' stamp: ''!
acquireResource: resourceName
        | anEvent |
        self timeStamp.
        DataFile nextPutAll: ' wants to serve for '.
        DataFile nextPutAll: resourceName.
        anEvent := super acquireResource: resourceName.
        self timeStamp.
        DataFile nextPutAll: ' can serve '.
        anEvent resource printOn: DataFile.
        ^anEvent! !

!EventMonitor methodsFor: 'scheduling' stamp: ''!
finishUp
        super finishUp.
        self timeStamp.
        DataFile nextPutAll: ' exits '.! !

!EventMonitor methodsFor: 'task language' stamp: ''!
holdFor: aTimeDelay
        self timeStamp.
        DataFile nextPutAll: ' holds for '.
        aTimeDelay printOn: DataFile.
        super holdFor: aTimeDelay! !

!EventMonitor methodsFor: 'initialization' stamp: ''!
initialize
        super initialize.
        self setLabel! !

!EventMonitor methodsFor: 'accessing' stamp: ''!
label
        ^label! !

!EventMonitor methodsFor: 'printing' stamp: ''!
printOn: aStream
        self class name printOn: aStream.
        aStream space.
        aStream nextPutAll: self label! !

!EventMonitor methodsFor: 'task language' stamp: ''!
produce: amount ofResource: resourceName
        self timeStamp.
        DataFile nextPutAll: ' produces '.
        amount printOn: DataFile.
        DataFile nextPutAll: ' of ', resourceName.
        super produce: amount ofResource: resourceName! !

!EventMonitor methodsFor: 'task language' stamp: ''!
produceResource: resourceName
        self timeStamp.
        DataFile nextPutAll: ' wants to get service as '.
        DataFile nextPutAll: resourceName.
        super produceResource: resourceName! !

!EventMonitor methodsFor: 'task language' stamp: ''!
release: aStaticResource
        self timeStamp.
        DataFile nextPutAll: ' releases '.
        aStaticResource amount printOn: DataFile.
        DataFile nextPutAll: ' of ', aStaticResource name.
        super release: aStaticResource! !

!EventMonitor methodsFor: 'task language' stamp: ''!
resume: anEvent
        self timeStamp.
        DataFile nextPutAll: ' resumes '.
        anEvent resource printOn: DataFile.
        super resume: anEvent! !

!EventMonitor methodsFor: 'accessing' stamp: ''!
setLabel
        Counter := Counter + 1.
        label := Counter printString.! !

!EventMonitor methodsFor: 'scheduling' stamp: ''!
startUp
        self timeStamp.
        DataFile nextPutAll: ' enters '.
        super startUp! !

!EventMonitor methodsFor: 'private' stamp: ''!
timeStamp
        DataFile cr.
        Simulation active time printOn: DataFile.
        DataFile tab.
        self printOn: DataFile! !

!SimulationObject class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SimulationObject methodsFor: 'task language' stamp: ''!
acquire: amount ofResource: resourceName 
	"Get the resource and then tell it to acquire amount of it.  Answers 
	an instance of StaticResource."

	^(Simulation active provideResourceFor: resourceName)
		acquire: amount withPriority: 0! !

!SimulationObject methodsFor: 'task language' stamp: ''!
acquire: amount ofResource: resourceName withPriority: priority 
	"Returns a StaticResource"
	^(Simulation active provideResourceFor: resourceName)
		acquire: amount withPriority: priority! !

!SimulationObject methodsFor: 'task language' stamp: ''!
acquireResource: resourceName
	"Returns a StaticResource"
	^(Simulation active provideResourceFor: resourceName) acquire! !

!SimulationObject methodsFor: 'simulation control' stamp: ''!
finishUp
	"Tell the simulation that the receiver is done with its tasks."
	Simulation active exit: self! !

!SimulationObject methodsFor: 'task language' stamp: ''!
holdFor: aTimeDelay
	Simulation active delayFor: aTimeDelay! !

!SimulationObject methodsFor: 'initialization' stamp: ''!
initialize
	"Do nothing.  Subclasses will initialize instance variables."

	^self! !

!SimulationObject methodsFor: 'task language' stamp: ''!
inquireFor: amount ofResource: resourceName
	^(Simulation active provideResourceFor: resourceName) amountAvailable >= amount! !

!SimulationObject methodsFor: 'task language' stamp: ''!
numberOfProvidersOfResource: resourceName
	| resource |
	resource := Simulation active provideResourceFor: resourceName.
	resource serversWaiting
		ifTrue: [^resource queueLength]
		ifFalse: [^0]! !

!SimulationObject methodsFor: 'task language' stamp: ''!
numberOfRequestersOfResource: resourceName
	| resource |
	resource := Simulation active provideResourceFor: resourceName.
	resource customersWaiting
		ifTrue: [^resource queueLength]
		ifFalse: [^0]! !

!SimulationObject methodsFor: 'task language' stamp: ''!
produce: amount ofResource: resourceName
	Simulation active produce: amount of: resourceName! !

!SimulationObject methodsFor: 'task language' stamp: ''!
produceResource: resourceName
	^(Simulation active provideResourceFor: resourceName) producedBy: self! !

!SimulationObject methodsFor: 'task language' stamp: ''!
release: aStaticResource
	^aStaticResource release! !

!SimulationObject methodsFor: 'task language' stamp: ''!
resourceAvailable: resourceName
	"Does the active simulaton have a resource with this attribute available?"
	^Simulation active includesResourceFor: resourceName! !

!SimulationObject methodsFor: 'task language' stamp: ''!
resume: anEvent
	^anEvent resume! !

!SimulationObject methodsFor: 'simulation control' stamp: ''!
startUp
	Simulation active enter: self.
	"First tell the simulation that the receiver is beginning to do my tasks."
	self tasks.
	self finishUp! !

!SimulationObject methodsFor: 'task language' stamp: ''!
stopSimulation
	Simulation active finishUp! !

!SimulationObject methodsFor: 'simulation control' stamp: ''!
tasks
	"Do nothing.  Subclasses will schedule activities."
	^self! !

SimulationObject subclass: #Visitor
	instanceVariableNames: 'entryTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-SimST80Demos'!

!Visitor methodsFor: 'access' stamp: ''!
entryTime
   ^entryTime.! !

!Visitor methodsFor: 'class initialization' stamp: ''!
initialize
        super initialize.
        entryTime := Simulation active time.! !

!Visitor methodsFor: 'access' stamp: ''!
tasks
        self holdFor:
                (Normal mean: 20 deviation: 5) next.! !

Object subclass: #SimulationObjectRecord
	instanceVariableNames: 'entranceTime duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mass-Statistics'!

!SimulationObjectRecord methodsFor: 'accessing' stamp: ''!
duration
        ^duration! !

!SimulationObjectRecord methodsFor: 'accessing' stamp: ''!
entrance
        ^entranceTime! !

!SimulationObjectRecord methodsFor: 'accessing' stamp: ''!
entrance: currentTime
        entranceTime := currentTime! !

!SimulationObjectRecord methodsFor: 'accessing' stamp: ''!
exit
        ^entranceTime + duration! !

!SimulationObjectRecord methodsFor: 'accessing' stamp: ''!
exit: currentTime
        duration := currentTime - entranceTime! !

!SimulationObjectRecord methodsFor: 'printing' stamp: ''!
printOn: aStream
        entranceTime printOn: aStream.
        aStream tab.
        duration printOn: aStream! !
ProbabilityDistribution initialize!
